package ru.dbotthepony.kstarbound.server.network

import io.netty.channel.ChannelHandlerContext
import org.apache.logging.log4j.LogManager
import ru.dbotthepony.kstarbound.network.Connection
import ru.dbotthepony.kstarbound.network.ConnectionSide
import ru.dbotthepony.kstarbound.network.ConnectionType
import ru.dbotthepony.kstarbound.network.IServerPacket
import ru.dbotthepony.kstarbound.network.packets.HelloPacket
import ru.dbotthepony.kstarbound.server.StarboundServer
import java.util.*
import kotlin.properties.Delegates

// server -> client
class ServerConnection(val server: StarboundServer, type: ConnectionType) : Connection(ConnectionSide.SERVER, type, UUID(0L, 0L)) {
	override fun channelRead(ctx: ChannelHandlerContext, msg: Any) {
		if (msg is IServerPacket) {
			try {
				msg.play(this)
			} catch (err: Throwable) {
				LOGGER.error("Failed to read serverbound packet $msg", err)
				disconnect(err.toString())
			}
		} else {
			LOGGER.error("Unknown serverbound packet type $msg")
			disconnect("Unknown serverbound packet type $msg")
		}
	}

	override var player: ServerPlayer by Delegates.notNull()
		private set

	override fun inGame() {
		server.playerInGame(player)
	}

	override fun onHelloReceived(helloPacket: HelloPacket) {
		player = ServerPlayer(this)
	}

	companion object {
		private val LOGGER = LogManager.getLogger()
	}
}
